/******************************************************************************* 
Power supply projekt
Zdroje: Arduino Core for ATMega 1284P by JChristensen https://github.com/JChristensen/mighty-1284p/tree/v1.6.3
Programovani: 
  Nejprve vypalit bootloader (Arduino as ISP, ISP programator pres 6 pinovy konektor, board Bobuino and Skinny Bob 16 MHz using Optiboot
    Odpojene napajeni ze site!!!
  S vypalenym bootloaderem lze prepnout na AVRISP MKII a uploadovat primo pres usb stiskem Upload
    Nutnost napajeni ze site - ted uz neni napajeno z USB
*******************************************************************************/
  /* KNIHOVNY */
#include <SPI.h>                            //Potrebne pro komunikaci s IO Expanderem, DAC i ADC
#include <MCP23S17.h>                       //Knihovna komunikace s IO Expanderem
#include <LiquidCrystal_I2C.h>              //LCD displej
#include <EEPROM.h>                         //Ukladani konstant do EEPROM
#include <OneWire.h>                        //Knihovna OneWire komunikace
#include <DallasTemperature.h>              //Knihovna Dallas Teplotnich senzoru (vcetne DS18B20)

/********************************************************/
  /* DEFINICE - MIKROKONRTOLER ATMEGA 1284P */
//IO Expander piny - piny GPIO B banky IO Expanderu!
#define Row1  1
#define Row2  2
#define Row3  3
#define Row4  4
#define Col1  5
#define Col2  6
#define Col3  7
#define Col4  8

//IO Expander - HW interrupt pin na MCU
#define IntKeypad  6                        //HW interrupt pro klavesnici (digital 6)
#define IntButtons 3                        //HW interrupt pro tlacitka

//IO Expander - SPI periferie
#define CS_EXP 10

//Rotary encoder
#define rotA  15
#define rotB  14
#define rotBT 16
#define INPUT_TIMEOUT_PERIOD  5000          //Doba do timeoutu zadavani keypadem/rot. enkoderem [ms]
#define RELEASE_CHECK_PERIOD  5             //Perioda kontroly pusteni tlacitek/keypadu [ms]

//Vetrak - adresa mista ulozeni v nevolatilni EEPROM pameti
#define FAN_PIN 8                           //PWM enabled pin 
#define FAN_CFG_ADDR 299                    //EEPROM adresa ulozeni rychlosti ventilatoru

//SPI periferie - ADC
#define CS_ADC1 21
#define CS_ADC2 19

//SPI periferie - DAC
#define CS_DAC1 20
#define CS_DAC2 18

//Piezo buzzer
#define PiezoPIN 30                         //PWM enabled pin
#define SOUND_CFG_ADDR 300                  //EEPROM adresa ulozeni povoleni zvukove signalizace
#define SHORT_BEEP_DUR 1                    //Trvani kratkeho pipnuti [ms]    //5 ms je uz docela tiche, vhodne na rot. enkoder? mozna jeste trochu mene... TBD
#define BURST_BEEP_DUR 25                   //Zvuk zamitnuti [ms]
#define LONG_BEEP_DUR  100                  //Varovny zvuk [ms]

//Teplotni senzory
#define ONE_WIRE         9                  //OneWire sbernice - teplota
#define AMBIENT_TEMP     1                  //Ambientni teplotu snima senzor s indexem 0 
#define HEATSINK1_TEMP   0                  //Teplotu heatsinku Ch1 snima senzor s indexem 1
#define HEATSINK2_TEMP   2                  //Teplotu heatsinku Ch2 snima senzor s indexem 2
#define TEMP_MEAS_PERIOD 5000               //Perioda mereni teploty a nasledneho nastaveni vetraku [ms]

//Vystupni rele kanalu
#define CH1_RELAY 28                        // !!! TEST - DEBUGOVACI PINY !!!
#define CH2_RELAY 29                        // !!! TEST - DEBUGOVACI PINY !!!

//Signaly proudove limitace
#define CH1_CLIM 24                         // !!! TEST - DEBUGOVACI PINY !!!
#define CH2_CLIM 25                         // !!! TEST - DEBUGOVACI PINY !!!
#define CLIM_CHECK_PERIOD 25                //Perioda kontroly proudove limitace [ms]

//Spare debugovaci piny
#define SP33  17
#define SP40  4
#define SP41  5
#define SP43  7
#define SP23  26
#define SP24  27
#define SP11  2 
#define CH2_EN_SP16  31                     //MISO CH2 enable (gate passgate tranzistoru, normal '0')

//Obnovovaci perioda LCD displeje [ms]
#define REFRESH_PERIOD  250

//Mereni U,I
#define AVG_SAMPLES 4                       //Pocet vzorku pro prumerovani merenych hodnot
#define MEASUREMENT_PERIOD 50               //Merici perioda [ms]

//Kalibrace U a I
#define U_CH1_CALIB_START_ADDR  10          //Adresa v EEPROM od ktere jsou ulozeny kalibracni udaje kanalu 1 a nasledne kanalu 2 - napeti
#define U_CH1_CALIB_END_ADDR    41
#define U_CH2_CALIB_START_ADDR  42
#define U_CH2_CALIB_END_ADDR    73
#define I_CH1_CALIB_START_ADDR  74          //Adresa v EEPROM od ktere jsou ulozeny kalibracni udaje kanalu 1 a nasledne kanalu 2 - proud
#define I_CH1_CALIB_END_ADDR    105
#define I_CH2_CALIB_START_ADDR  106
#define I_CH2_CALIB_END_ADDR    137

#define DAC_U_CH1_CALIB_START_ADDR  138     //Adresa v EEPROM od ktere jsou ulozeny kalibracni udaje kanalu 1 a nasledne kanalu 2 - napeti
#define DAC_U_CH1_CALIB_END_ADDR    169
#define DAC_U_CH2_CALIB_START_ADDR  170
#define DAC_U_CH2_CALIB_END_ADDR    201

//Pomocne definice
#define NO_PARSE 0
#define PARSE 1
#define NO_PARSE_NO_LCD 2
#define NO_CALIB 0
#define USE_CALIB 1
/********************************************************/
  /* GLOBALNI PROMENNE */
  //IO Expander - klavesnice  
volatile uint8_t row_read, col_read, dummy; //Cteni klavesy
volatile uint8_t row_konv, col_konv;        //Souradnice stiskleho znaku v array 'klavesy'
char klavesy [4][4]={ '1', '2', '3', 'A',   //Rozvrzeni klavesnice 4x4
                      '4', '5', '6', 'B',
                      '7', '8', '9', 'C',
                      '.', '0', '#', 'D'};
volatile char pressed_key = 0;              //Pri stisku klavesy je sem v ISR ulozen vysledek   
volatile bool new_key = 0;                  //Priznak nastaveny pri stisku klavesy v ISR
char keypad_input_string[7] = {'0'};        //String, nastaveni proudu nebo napeti
uint8_t var_select = 0;                     //Vyber nastaveni - 0=nic, 1=Ch1.V, 2=Ch1.I, 3=Ch2.V, 4=Ch2.I
uint8_t input_len = 0;                      //Pocet zadanych znaku
uint8_t point_pos = 0;                      //Pozice desetinne tecky
bool keypad_numeric_input_ena = 0;          //Flag zda je povoleno zadavat na keypadu cisla
bool rot_enc_input_ena = 0;                 //Flag zda je povoleno upravovat hodnotu pomoci rot. enc.
unsigned long lastKeypadCheck;
unsigned long lastKeypadPress; 
MCP expander(0, CS_EXP);                    //Inicializuje objekt 'expander" [MCP23S17] s adresou [0] a slave-selectem na MCU pinu [10]
  
  //IO Expander - tlacitka
volatile bool new_btnpress = 0;             //Flag nove detekce stiskleho tlacitka
volatile uint8_t pressed_btn = 0;           //Stav registru s tlacitky (bity 7..2 jsou tlacitka, 1..0 jsou rele outputy) //bit 7=lock, 6=menu, 5=lim, 4=both out, 3=out2, 2=out1, 1 a 0 = vystupy 
unsigned long lastButtonCheck;
bool show_limits = 0;                       //Flag prepinani LCD zobrazeni - ukazovani aktualnich hodnot/ukazovani nastavenych hodnot
  
  //LCD displej
LiquidCrystal_I2C lcd(0x27, 20, 4);         //Inicializace LCD displeje 20x4
byte zamek[8] = {                           //Symbol zamku
  0b01110,
  0b10001,
  0b10001,
  0b11111,
  0b11011,
  0b11011,
  0b11111,
  0b00000
};
byte stupenC[8] = {                         //Symbol °C
  0b00110,
  0b01001,
  0b01001,
  0b00110,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};
unsigned long lastLCDrefresh;               //Timestamp obnovovani hodnot na LCD displeji
typedef struct LCD_prev_values {            //Pamet hodnot ktere byly na LCD displeji zobrazeny v prechozim obnoveni (volani refreshLCD)
  float Ch1_Vread = 0;
  float Ch1_Iread = 0;
  float Ch2_Vread = 0;
  float Ch2_Iread = 0;
} display_prev_values;
display_prev_values LCD_values;
  
  //Rotacni enkoder
volatile bool rotA_Last, rotBT_Last, rotA_State, rotB_State, rotBT_State;   
int change_delitel = 10;
unsigned long lastRotaryChange;

  //Nastaveni kanalu - struktura
typedef struct Channel {
  float Vset  = 0.0;                          //Nastavene napeti/proud
  float Iset  = 1.023;
  float Vread = 0.0;                          //Zmerene napeti/proud
  float Iread = 0.0;
  bool output_en = 0;                         //Stav vystupniho rele (1 = aktivni vystup)  
  bool CLIM = 0;                              //Detekce stavu proudoveho omezeni
} kanal;                                      
kanal Ch1, Ch2;                               //Deklarace promennych Ch1, Ch2 - typu struktura Channel

  //Konstanty a nastaveni zdroje
const float V_per_bit = 0.01;                 //10.0 mV/bit
const float A_per_bit = 0.00025;              //0.25 mA/bit
const int   V_max = 30;                       //maximalni vystupni napeti
const float A_max = 1.023;                    //maximalni vystupni proud
const int DAC_code_low_limit = 20;            //minimalni realne nastavitelny kod DA prevodniku ktery se presne prenese na vystup PSU (limitovano napajenim OZ)
bool value_lock = 0;                          //flag uzamceni zmen hodnot, 0 = lze menit hodnoty U,I
unsigned long lastCLIMCheck = 0;
  //Pamet poslednich namerenych hodnot pro prumerovani
float last_samples[4][AVG_SAMPLES] = {0};     //Array poslednich AVG_SAMPLES vzorku, last_samples[U1, I1, U2, I2][x-AVG_SAMPLES,....x-2 ,x-1 ,x(nejnovejsi sample)]  
unsigned long lastMeasurement = 0;            //Timestamp posledniho mereni U, I

  //Kalibrace U, I
int u_calib_table[2][32]={0};                 //Tabulka kalibracnich konstant pro kalibraci ADC, nacte se z EEPROM pri zapnuti zdroje
int i_calib_table[2][32]={0};  
int DAC_u_calib_table[2][32]={0};             //Tabulka kalibracnich konstant pro kalibraci DAC, nacte se z EEPROM pri zapnuti zdroje            
  
  //Ventilator - PWM rizeny
uint8_t USER_FAN_SPEED = 255;  
uint8_t FAN_SPEED = 255;  

  //Aktualni timestamp
unsigned long currentMillis;

//OneWire inicializace - teplotni cidla Dallas DS18B20
OneWire oneWire(ONE_WIRE);
DallasTemperature temp_sensors(&oneWire);
float temperatures[3] = {23};                 //Array teplot ze vsech 3 teplotnich senzoru [°C]
unsigned long lastTempCheck = 0;

//Piezo buzzer
bool sound_muted = 0;                         //Flag ztlumeni zvukove signalizace, 0 = zvuk povolen
uint8_t CLIM_signalized = 10;

//Seriova komunikace
String IN_String;                             //String se vstupy ze serioveho portu
bool IN_String_Complete;                      //Flag ukonceni vstupniho stringu 
                             
  /******************************************************************************/ 
    /* SETUP */
  void setup() {
    //Seriova komunikace - zahajeni
  Serial.begin(115200);
  IN_String.reserve(200);                     //Pro vstupni string je reservovano 200B v pameti
    
    //CH2 MISO passgate - defaultne vypnuto
  pinMode(CH2_EN_SP16, OUTPUT);
  digitalWrite(CH2_EN_SP16, LOW);
  
    //Pocatecni nastaveni vystupnich rele
  pinMode(CH1_RELAY, OUTPUT);               
  pinMode(CH2_RELAY, OUTPUT);
  digitalWrite(CH1_RELAY, Ch1.output_en);
  digitalWrite(CH2_RELAY, Ch2.output_en);
  
    //Signaly proudove limitace
   pinMode(CH1_CLIM, INPUT);
   pinMode(CH2_CLIM, INPUT);  
    
    //SPI setup
  ADC_init();
  DAC_init();

    //Piezo
  pinMode(PiezoPIN, OUTPUT);
  digitalWrite(PiezoPIN, LOW);
  sound_muted = EEPROM.read(SOUND_CFG_ADDR);  //Nacteni posledniho stavu z pameti
  
    //Rotary encoder
  pinMode(rotA, INPUT);
  pinMode(rotB, INPUT);
  pinMode(rotBT, INPUT);
  rotA_Last =  !(digitalRead(rotA));          //Pocatecni stavy vstupu A, slouzi pro detekci otoceni 
  rotBT_Last = !(digitalRead(rotBT)); 
    
    //Setup IO Expanderu
  pinMode(IntKeypad, INPUT_PULLUP);
  pinMode(IntButtons, INPUT_PULLUP);
  expander.begin();
  IOexpander_setup();

    //Stav vetraku - rychlost (nacteni posledniho stavu z pameti)
  USER_FAN_SPEED = EEPROM.read(FAN_CFG_ADDR);  

    //Spusteni ventilatoru podle posledni ulozene hodnoty
  pinMode(FAN_PIN, OUTPUT);
  digitalWrite(FAN_PIN, 0);
  delay(10);
  for(int i=0; i<127; i++) {                  //Postupne nabihani vetraku - nejprve zajisteni rozjeti, pak do nastavene hodnoty
    analogWrite(FAN_PIN, i);
    delay(5);
  }
  analogWrite(FAN_PIN, USER_FAN_SPEED);
  
    //Nacteni kalibracnich konstant - oba kanaly 
  for(int i=0; i<2; i++) {
    int mem_offset = 0;
    if(i == 0) mem_offset = U_CH1_CALIB_START_ADDR;
    else       mem_offset = U_CH2_CALIB_START_ADDR;
    for(int j=0; j<32; j++) {
      int temp = EEPROM.read(j+mem_offset);
      if(temp >= 100) temp = 100-temp;
      u_calib_table[i][j] = temp;             //Napeti
    }
  }
  for(int i=0; i<2; i++) {
    int mem_offset = 0;
    if(i == 0) mem_offset = I_CH1_CALIB_START_ADDR;
    else       mem_offset = I_CH2_CALIB_START_ADDR;
    for(int j=0; j<32; j++) {
      int temp = EEPROM.read(j+mem_offset);
      i_calib_table[i][j] = temp;             //Proud
    }
  }
  for(int i=0; i<2; i++) {
    int mem_offset = 0;
    if(i == 0) mem_offset = DAC_U_CH1_CALIB_START_ADDR;
    else       mem_offset = DAC_U_CH2_CALIB_START_ADDR;
    for(int j=0; j<32; j++) {
      int temp = EEPROM.read(j+mem_offset);
      if(temp >= 100) temp = 100-temp;
      DAC_u_calib_table[i][j] = temp;         //DAC kalibrace
    }
  }

    //DAC pocatecni konfigurace - limit 1,024 A a napeti 0V
  DAC_SET(1, 'I', 4095, USE_CALIB);
  DAC_SET(2, 'I', 4095, USE_CALIB);
  DAC_SET(1, 'U', 0, USE_CALIB); 
  DAC_SET(2, 'U', 0, USE_CALIB);

    //LCD displej
  lcd_Start_Config();

    //Teplotni senzory
  temp_sensors.begin();
  temp_sensors.setResolution(9);                //Nastaveni rozliseni DS18B20 - 9 až 12 bitu, 10 bitu = 0.25°C a konverze pod 200ms, 9 bitu = 0.5°C a konverze pod 100 ms  

    //Aktualni timestamp
  currentMillis = millis();
  lastKeypadCheck  = currentMillis;
  lastKeypadPress  = currentMillis;
  lastButtonCheck  = currentMillis; 
  lastRotaryChange = currentMillis;
  lastMeasurement = currentMillis;
}

/******************************************************************************/ 
  /* MAIN LOOP */
void loop() {
  /* Cekani uvolneni klavesy - Pokus o clearnuti interrupt registru vyctenim GPIO registru jednou za 10 ms*/
  currentMillis = millis();               //Aktualni timestamp
  
  //Obnoveni hodnot na LCD displeji
  perform_measurement();                  //Provede mereni U,I jednou za MEASUREMENT_PERIOD
  refreshLCD();                           //Obnoveni displeje jednou za REFRESH_PERIOD

  //Kontrola proudove limitace
  check_CLIM();

  //Mereni teploty a nastaveni rychlosti ventilatoru
  temperature_and_fans();
  
  //Keypad - snaha o clearnuti interruptu [jen do uspesneho clearu], kontrola timeoutu zadavani
  check_keypad_release();
  keypad_timeout_check();
  
  //Tlacitka - snaha o clearnuti interruptu [jen do uspesneho clearu]
  check_btn_release();

  //Zpracovani nove stiskle klavesy keypadu
  if(new_key == 1) {
    new_key = 0;
    keypad_Zpracovani();
  } //End new key
  
  //Zpracovani nove stiskleho tlacitka
  if(new_btnpress == 1) {
    button_Zpracovani();
  }//End new_btnpress

  //Zpracovani vstupu ze serioveho portu
  if (IN_String_Complete == 1) {
    Parse_String();                           //Parsovani stringu
    IN_String = "";                           //Vymazani stringu a flagu
    IN_String_Complete = 0;
  }

  //Kontrola rotacniho encoderu - detekce otaceni, stisku tlacitka a timeoutu
  int rot_output = rotary_change_check();
  rotary_Zpracovani(rot_output);            //Mela by to byt posledni funkce ktera kresli na LCD displej - dulezita poloha kurzoru    
  rotary_timeout_check();
} //End Main Loop

/******************************************************************************/


/********************************************************/  
      /* SERIOVA KOMUNIKACE - PRIJEM DAT Z PC */
/********************************************************/ 
//serialEvent funkce je volana vzdy po ukonceni pruchodu Main() loopem
void serialEvent() {
  while (Serial.available()) {          //Dostupny minimalne jeden byte v bufferu
    char inChar = (char)Serial.read();  //Vycteni noveho bytu a pridani do vstupniho stringu
    IN_String += inChar;
    if (inChar == '\n') {               //Pokud je prichozi char symbol noveho radku "\n", nastav flag a zpracuj v mainu
      IN_String_Complete = 1;
    }
  }    
}
