/********************************************************/  
  /* DA PREVODNIK - NASTAVENI NAPETI A LIMITU PROUDU */
  /* MCP4822 Digital - Analog prevodnik 12bit 2 kanaly */
/********************************************************/  
void SPI_16bit_write(uint8_t CSpin, word data)  {
  digitalWrite(CSpin, LOW);  //Chip select LOW
  SPI.transfer16(data);
  digitalWrite(CSpin, HIGH); //Chip select HIGH  
}

//////////////////////////////////////////////////////////////////////////////
/* Sestavi 16bit datovou strukturu pro komunikaci s MCP4822 12bit DAC prevodnikem */
void DAC_konfigurace(uint8_t PSU_channel, char DAC_channel, uint8_t gain, bool power, int data){
  //PSU_channel 1/2, DAC_channel I/U, gain 1/2, output on(1)/off(0), data = 0-4095
  //Pretvareni zadanych udaju do 16bit wordu pro odeslani
  word SPI_data = 0x0000;                         //16-bit data
  //bit 16 - channel
    if(DAC_channel == 'I')  SPI_data &= ~(1<<15); //I = DAC CH A - nastav 0
    else                    SPI_data |= (1<<15);  //U = DAC CH B - nastav 1
  //bit 15 - dont care
  //////////////////////////////////////////
  //bit 14 - gain
    if(gain == 1) SPI_data |= (1<<13);          //GAIN 1x - nastav 1
    else          SPI_data &= ~(1<<13);         //GAIN 2x - nastav 0
  //bit 13 - shutdown
    if(power == 1)  SPI_data |= (1<<12);        //OUTPUT ON - nastav 1
    else            SPI_data &= ~(1<<12);       //OUTPUT OFF - nastav 0
  //bit 12 - 0 - data
    if(data >= 0 && data < 4096) {              //DATA - MSB first
        SPI_data |= data;
      }
    else SPI_data |= 0x000;
  //Kanal zdroje - Ch1 / Ch2  
  if(PSU_channel == 1)  SPI_16bit_write(CS_DAC1, SPI_data);
  else                  SPI_16bit_write(CS_DAC2, SPI_data);

  return;
}
//////////////////////////////////////////////////////////////////////////////
/* Pocatecni nastaveni pinu */
void DAC_init(void) {
  pinMode(CS_DAC1, OUTPUT);
  digitalWrite(CS_DAC1, HIGH);
  pinMode(CS_DAC2, OUTPUT);
  digitalWrite(CS_DAC2, HIGH);
  return;
}
//////////////////////////////////////////////////////////////////////////////
/* Sestavi 16bit datovou strukturu pro komunikaci s MCP4822 12bit DAC prevodnikem */
void DAC_SET(uint8_t PSU_channel, char variable, int data, bool calibration){
  //PSU_channel 1/2, variable I/U, data = 0-4095, calibration NO_CALIB/USE_CALIB [0/1]
  if(calibration == NO_CALIB) {                                 //Odesle zadana data do DAC bez ohledu na kalibraci
    DAC_konfigurace(PSU_channel, variable, 2, 1, data);
    return;  
  }                                                             //Vycteni kalibracni konstanty a uprava hodnoty "data" pred odeslanim do DAC
  int calib_data = DA_kompenzace(PSU_channel, variable, data);
  DAC_konfigurace(PSU_channel, variable, 2, 1, calib_data);
  return;
}
//////////////////////////////////////////////////////////////////////////////
