//////////////////////////////////////////////////////////////////////////////
//Ziskani teploty ve stupnich celsia ze vsech 3 senzoru DS18B20, pokud neni pripojen, vraci hodnotu DEVICE_DISCONNECTED_C
void get_temperatures(void) {
  temp_sensors.requestTemperatures();                   //Globalni pozadavek na zmereni teplot (pro vsechny teplotni senzory)
  for(int i=0; i<3; i++){
    temperatures[i] = temp_sensors.getTempCByIndex(i);  //Prvni, druhy a treti senzor na OW sbernici
  }  
  return;
}
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//Nastaveni rychlosti ventilatoru dle zmerene teploty nebo prani uzivatele
void setFans(void) {
  //v globalni promenne USER_FAN_SPEED je rychlost ventilatoru, kterou si preje uzivatel. Toto nastaveni ma nejnizsi prioritu
  const int fan_curve_ambient[4][2]  =  {{-40,0},{33,0},{35,127},{40,255}};   //Nahledova tabulka hodnot {{[°C][fan speed]},[°C][fan speed]}...} pro Ambientni teplotu
  const int fan_curve_heatsink[4][2] =  {{-40,0},{47,0},{50,127},{80,255}};   //Nahledova tabulka hodnot {{[°C][fan speed]},[°C][fan speed]}...} pro Heatsink teplotu

  //Rovnice linearniho splajnu pro bod [x,y] mezi body [x1, y1] a [x2, y2]
  //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
  uint8_t final_setting = 0;
  //Ambientni teplota
  if(temperatures[AMBIENT_TEMP] != DEVICE_DISCONNECTED_C) {                   //Kontrola zda neni odpojen teplotni senzor 
    if(temperatures[AMBIENT_TEMP] <= fan_curve_ambient[0][0] || temperatures[AMBIENT_TEMP] >= fan_curve_ambient[3][0]) {
      //Kontrola zda neni teplota zcela mimo limity
      final_setting = 255;
    }
    else {
      for(int i=1; i<4; i++){                                                 //Hledam mezi kterymi dvema body v lookup tabulce jsem
        if(fan_curve_ambient[i-1][0] <= temperatures[AMBIENT_TEMP] && temperatures[AMBIENT_TEMP] < fan_curve_ambient[i][0]) {
          //Linearni interpolace
          //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
          float interp_speed = fan_curve_ambient[i-1][1]+(((fan_curve_ambient[i][1]-fan_curve_ambient[i-1][1])/(fan_curve_ambient[i][0]-fan_curve_ambient[i-1][0]))*(temperatures[AMBIENT_TEMP]-fan_curve_ambient[i-1][0]));
          if(interp_speed > USER_FAN_SPEED) final_setting = interp_speed;     //Pokud je uzivatelsky nastavena rychlost mensi nez rychlost ventilatoru ktera odpovida teto teplote podle ulozene krivky, ignoruj uzivatelske nastaveni
          else                              final_setting = USER_FAN_SPEED;   //Pokud je uzivatelsky nastavena rychlost vetsi nez pro tuto teplotu odpovidajici rychlost dle krivky, ponech uzivatelske nastaveni
          break;
        }
      }
    }//end else out of bounds 
  }//End Ambient
  
  //Teplota Heatsink1
  if(temperatures[HEATSINK1_TEMP] != DEVICE_DISCONNECTED_C) {                 //Kontrola zda neni odpojen teplotni senzor 
    if(temperatures[HEATSINK1_TEMP] <= fan_curve_heatsink[0][0] || temperatures[AMBIENT_TEMP] >= fan_curve_heatsink[3][0]) {
      //Kontrola zda neni teplota zcela mimo limity
      final_setting = 255;
    }
    else {
      for(int i=1; i<4; i++){                                                 //Hledam mezi kterymi dvema body v lookup tabulce jsem
        if(fan_curve_heatsink[i-1][0] <= temperatures[HEATSINK1_TEMP] && temperatures[HEATSINK1_TEMP] < fan_curve_heatsink[i][0]) {
          //Linearni interpolace
          //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
          float interp_speed = fan_curve_heatsink[i-1][1]+(((fan_curve_heatsink[i][1]-fan_curve_heatsink[i-1][1])/(fan_curve_heatsink[i][0]-fan_curve_heatsink[i-1][0]))*(temperatures[HEATSINK1_TEMP]-fan_curve_heatsink[i-1][0]));
          if(interp_speed > USER_FAN_SPEED && interp_speed > final_setting) final_setting = interp_speed;     //Pokud je uzivatelsky nastavena rychlost mensi nez rychlost ventilatoru ktera odpovida teto teplote podle ulozene krivky, ignoruj uzivatelske nastaveni. Zaroven nesmi byt mensi nez rychlost nastavena v predchozim kroku
          else if(USER_FAN_SPEED > final_setting)                           final_setting = USER_FAN_SPEED;   //Pokud je uzivatelsky nastavena rychlost vetsi nez pro tuto teplotu odpovidajici rychlost dle krivky, ponech uzivatelske nastaveni. Zaroven nesmi byt mensi nez rychlost nastavena v predchozim kroku.
          break;
        }
      }
    }//end else out of bounds 
  }//End Heatsink1

  //Teplota Heatsink2
  if(temperatures[HEATSINK2_TEMP] != DEVICE_DISCONNECTED_C) {                 //Kontrola zda neni odpojen teplotni senzor 
    if(temperatures[HEATSINK2_TEMP] <= fan_curve_heatsink[0][0] || temperatures[AMBIENT_TEMP] >= fan_curve_heatsink[3][0]) {
      //Kontrola zda neni teplota zcela mimo limity
      final_setting = 255;
    }
    else {
      for(int i=1; i<4; i++){                                                 //Hledam mezi kterymi dvema body v lookup tabulce jsem
        if(fan_curve_heatsink[i-1][0] <= temperatures[HEATSINK2_TEMP] && temperatures[HEATSINK2_TEMP] < fan_curve_heatsink[i][0]) {
          //Linearni interpolace
          //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
          float interp_speed = fan_curve_heatsink[i-1][1]+(((fan_curve_heatsink[i][1]-fan_curve_heatsink[i-1][1])/(fan_curve_heatsink[i][0]-fan_curve_heatsink[i-1][0]))*(temperatures[HEATSINK2_TEMP]-fan_curve_heatsink[i-1][0]));
          if(interp_speed > USER_FAN_SPEED && interp_speed > final_setting) final_setting = interp_speed;     //Pokud je uzivatelsky nastavena rychlost mensi nez rychlost ventilatoru ktera odpovida teto teplote podle ulozene krivky, ignoruj uzivatelske nastaveni. Zaroven nesmi byt mensi nez rychlost nastavena v predchozim kroku
          else if(USER_FAN_SPEED > final_setting)                           final_setting = USER_FAN_SPEED;   //Pokud je uzivatelsky nastavena rychlost vetsi nez pro tuto teplotu odpovidajici rychlost dle krivky, ponech uzivatelske nastaveni. Zaroven nesmi byt mensi nez rychlost nastavena v predchozim kroku.
          break;
        }
      }
    }//end else out of bounds 
  }//End Heatsink2

  FAN_SPEED = final_setting;                                                  //Nakonec se jako rychlost ventilatoru pouzije nejvyssi pozadovana rychlost
  analogWrite(FAN_PIN, FAN_SPEED);
  return;
}
//////////////////////////////////////////////////////////////////////////////

//Zmereni teploty a nastaveni rychlosti ventilatoru 
void temperature_and_fans(void) {
  if(currentMillis - lastTempCheck >= TEMP_MEAS_PERIOD) {
    lastTempCheck = currentMillis;  
    get_temperatures();                                   //Zmereni teploty ze vsech tri senzoru
    setFans();                                            //Nastaveni rychlosti ventilatoru podle teploty a uzivatelskeho nastaveni
  }
  return;
}
//////////////////////////////////////////////////////////////////////////////
