/********************************************************/  
/* SERIOVA KOMUNIKACE - PARSOVANI PRIJATEHO STRINGU Z PC*/
/********************************************************/ 
void Parse_String(void) {
  String CMD_SET = "SET";
  String CMD_GET = "GET";
  String CMD_OUT = "OUT";
  if(IN_String.startsWith(CMD_SET)) {         //Byl prijaty command SET - nastaveni U nebo I
    Serial.println(" -> CMD SET");
    Parse_SET_CMD();
  }
  else if(IN_String.startsWith(CMD_GET)) {    //Byl prijaty command GET - vycteni U nebo I
    Serial.println(" -> CMD GET");
    Parse_GET_CMD();
  }
  else if(IN_String.startsWith(CMD_OUT)) {    //Byl prijaty command OUT - nastaveni vystupniho rele
    Serial.println(" -> CMD OUT");
    Parse_OUT_CMD();
  }
  else {
    Serial.println(" -> UNKNOWN CMD");
    Serial.println(IN_String);                //Neznamy prikaz - vratim to co bylo poslano (test loopback)
  }
  return;
}

/********************************************************/

/********************************************************/  
    /* SERIOVA KOMUNIKACE - ZPRACOVANI SET COMMANDU */
/********************************************************/ 
void Parse_SET_CMD(void) {                          //Slouzi k nastaveni U, I; priklad struktury: SET U1 12.34; ocekavana odpoved: OK
  char var = IN_String.charAt(4);                   //'U' nebo 'I'
  if(var != 'U' && var != 'I') {              
    Serial.println("VARIABLE ERR");
    return;
  }
  uint8_t chan = (IN_String.charAt(5))-48;          //Odecteni 48 pro prevod ASCII -> INT
  String str_value = IN_String.substring(7);        //Vykopiruje z prichoziho stringu pouze ciselnou hodnotu
  uint8_t str_len = str_value.length();
  if(str_len > 7) {
    Serial.println("RANGE ERR");
    return;
  }
  float value = str_value.toFloat();                //Prevod String->float
  if(str_len == 7 && var == 'I') {                  //toFloat orezava na dve desetinna mista, rucne dokazu na tri
    value = 1*(IN_String.charAt(7)-48) + 0.1*(IN_String.charAt(9)-48) + 0.01*(IN_String.charAt(10)-48) + 0.001*(IN_String.charAt(11)-48);  
  }
  if((var == 'U' && (value < 0 || value > V_max)) || (var == 'I' && (value < 0 || value > A_max))) {
    Serial.println("VALUE ERR");
    return;
  }
  //Serial.print("value: "); Serial.println(value);
  if(chan == 1) {
    if(var == 'U') {
      Ch1.Vset = value; DAC_SET(1, 'U', value/V_per_bit, USE_CALIB); /*DAC_konfigurace(1, 'U', 2, 1, value/V_per_bit);*/}
    else {
      Ch1.Iset = value; DAC_SET(1, 'I', value/A_per_bit, USE_CALIB); /*DAC_konfigurace(1, 'I', 2, 1, value/A_per_bit);*/}
  }
  else if(chan == 2) {
    if(var == 'U') {
      Ch2.Vset = value; DAC_SET(2, 'U', value/V_per_bit, USE_CALIB); /*DAC_konfigurace(2, 'U', 2, 1, value/V_per_bit);*/}
    else {
      Ch2.Iset = value; DAC_SET(2, 'I', value/A_per_bit, USE_CALIB); /*DAC_konfigurace(2, 'I', 2, 1, value/A_per_bit);*/}    
  }
  else {
    Serial.println("CH NUMBER ERR");
    return;    
  }
  Serial.println("OK");
  redrawLcd();                                        //Aktualizace hodnot na LCD
  return;
}
/********************************************************/

/********************************************************/  
    /* SERIOVA KOMUNIKACE - ZPRACOVANI GET COMMANDU */
/********************************************************/ 
void Parse_GET_CMD(void) {                          //Slouzi k vycteni U, I; priklad struktury: GET I1; ocekavana odpoved: I1 0.250 A
  char var = IN_String.charAt(4);                   //'U' nebo 'I'
  //Serial.print(" var: "); Serial.println(var);
  if(var != 'U' && var != 'I') {              
    Serial.println("VARIABLE ERR");
    return;
  }
  uint8_t chan = (IN_String.charAt(5))-48;          //Odecteni 48 pro prevod ASCII -> INT
  //Serial.print("chan: "); Serial.println(chan);
  String response = "";
  if(chan == 1) {
    if(var == 'U')
      response = "U1 "+String(Ch1.Vread, 2)+" V";   //Napeti na dve desetinna mista
    else
      response = "I1 "+String(Ch1.Iread, 3)+" A";   //Proud na tri desetinna mista
  }
  else if(chan == 2) {
     if(var == 'U')
      response = "U2 "+String(Ch2.Vread, 2)+" V";
    else
      response = "I2 "+String(Ch2.Iread, 3)+" A";    
  }
  else {
    Serial.println("CH NUMBER ERR");
    return;    
  }
  Serial.println(response);
  return;
}
/********************************************************/

/********************************************************/  
    /* SERIOVA KOMUNIKACE - ZPRACOVANI OUT COMMANDU */
/********************************************************/ 
void Parse_OUT_CMD(void) {                          //Slouzi k nastaveni vystupniho rele napajecich kanalu; priklad struktury: OUT 1 1
  uint8_t chan = (IN_String.charAt(4))-48;          //Odecteni 48 pro prevod ASCII -> INT
  uint8_t state = (IN_String.charAt(6))-48;         //Stav vystupu, 1 = zapnut, 0 = vypnut
  if(state != 0 && state != 1) {
    Serial.println("OUT STATE ERR");
    return;   
  }
  if(chan == 1) {
    Ch1.output_en = state; 
    digitalWrite(CH1_RELAY, Ch1.output_en);         //Zmena stavu kanalu 1
  }
  else if(chan == 2) {
    Ch2.output_en = state; 
    digitalWrite(CH2_RELAY, Ch2.output_en);         //Zmena stavu kanalu 2
  }
  else {
    Serial.println("CH NUMBER ERR");
    return;    
  }
  Serial.println("OK");
  redrawLcd();                                      //Aktualizace hodnot na LCD
  return;
}
