/********************************************************/  
          /* KOMPENZACE CHYB ADC PREVODNIKU */
/********************************************************/
/* Kompenzace gain error a offset error prevodniku ADC
    Vstupy  - AD_value = hodnota zmerena ADC 0-4095
            - PSU_channel = 1 nebo 2
            - ADC_channel = 'U' nebo 'I' 
    Vystup  - int hodnota kompenzovana kalibracimi konstantami (z u_calib_table[kanal 0-1][DAC code odstupnovany po 100 (20-3100) 0-31])
*/
int AD_komepnzace(int AD_value, uint8_t PSU_channel, char ADC_channel) {
  //Serial.println("_ _ _ _ _ _");
  //Serial.print("AD:"); Serial.print(AD_value); Serial.print("   CH:"); Serial.print(PSU_channel); Serial.print("   ADCH:"); Serial.println(ADC_channel);
  int result = 0;
  
    //Kompenzace napeti
  if(ADC_channel == 'U') {
    uint8_t index = AD_value/100;
    if(AD_value >= V_max/V_per_bit) {         //Vmax = 30V a V per bit = 0.01 => code 3000 je max, zastropovani
      result = AD_value-u_calib_table[PSU_channel-1][(uint8_t)V_max];       //Index [30] pokud Vmax = 30 a na teto pozici je ulozena kompenzace pro kod 3000
    }
    else if(AD_value <= DAC_code_low_limit) { //Limitace min. nastavitelneho napeti operacnim zesilovacem
      //Serial.print("LOW LIMIT CALIB CONSTANT:"); Serial.print(u_calib_table[PSU_channel-1][0]);
      if(AD_value-u_calib_table[PSU_channel-1][0] >= 0){
        result = AD_value-u_calib_table[PSU_channel-1][0];
      }
      else {
        result = 0;
      }
      //Serial.print("result:"); Serial.print(result); Serial.println(result);
    }
    else {                                    //Linearni interpolace
      //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
      int interp_comp = u_calib_table[PSU_channel-1][index]+(((u_calib_table[PSU_channel-1][index+1]-u_calib_table[PSU_channel-1][index])/((index+1)*100-index*100))*(AD_value-index*100));
      result = AD_value-interp_comp;        //Napr. DAC=100, ADC=96, delta=-4, korigovane ADC=96-(-4)=100  
      //Serial.print("x:"); Serial.print(AD_value); Serial.print("   x1:"); Serial.print(index*100); Serial.print("   x2:"); Serial.print((index+1)*100); Serial.print("   y1:"); Serial.print(u_calib_table[PSU_channel-1][index]); Serial.print("   y2:"); Serial.println(u_calib_table[PSU_channel-1][index+1]);
      //Serial.print("interp_comp:"); Serial.println(interp_comp);
    }
  }
    //Kompenzace proudu (nulovani)
  else if(ADC_channel == 'I') {
    int u_code = 0;
    uint8_t index = 0;
    
    //Zjisteni soucasneho vystupniho napeti, jeho meziprevod na kod(+kalibrace) a pak na index v tabulce
    if(PSU_channel == 1) {
      u_code = Ch1.Vread/V_per_bit;                       //30Vout / 0.01 = code 3000 
      u_code = DA_kompenzace(PSU_channel, 'U', u_code);   //doplneni o kalibracni konstanty
      index = u_code/100;                                 //30Vout / 0.01 = code 3000 /100 = index 30
    }
    else {
      u_code = Ch2.Vread/V_per_bit;                       //30Vout / 0.01 = code 3000
      u_code = DA_kompenzace(PSU_channel, 'U', u_code);   //doplneni o kalibracni konstanty
      index = u_code/100;                                 //30Vout / 0.01 = code 3000 /100 = index 30  
    }
    
    if(u_code >= V_max/V_per_bit) {                                     //Vmax = 30V a V per bit = 0.01 => code 3000 je max, zastropovani //Index [30] pokud Vmax = 30 a na teto pozici je ulozena kompenzace pro kod 3000
      if(AD_value-i_calib_table[PSU_channel-1][(uint8_t)V_max] >= 0)  result = AD_value-i_calib_table[PSU_channel-1][(uint8_t)V_max];
      else                                                            result = 0;
    }
    else if(u_code <= DAC_code_low_limit) {                             //Limitace min. nastavitelneho napeti operacnim zesilovacem
      if(AD_value-i_calib_table[PSU_channel-1][0] >= 0) result = AD_value-i_calib_table[PSU_channel-1][0];
      else                                              result = 0;
    }
    else {                                                              //Linearni interpolace
      //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
      int interp_comp = i_calib_table[PSU_channel-1][index]+(((i_calib_table[PSU_channel-1][index+1]-i_calib_table[PSU_channel-1][index])/((index+1)*100-index*100))*(u_code-index*100));
      if(AD_value-interp_comp >= 0) result = AD_value-interp_comp; 
      else                          result = 0; 
      //Serial.print("x:"); Serial.print(u_code); Serial.print("   x1:"); Serial.print(index*100); Serial.print("   x2:"); Serial.print((index+1)*100); Serial.print("   y1:"); Serial.print(i_calib_table[PSU_channel-1][index]); Serial.print("   y2:"); Serial.println(i_calib_table[PSU_channel-1][index+1]);
      //Serial.print("interp_comp:"); Serial.println(interp_comp);
    }
  }
  //Serial.print("Result:"); Serial.println(result);
  //Serial.println("_____________________________");
  if(result >= 0 && result <= 4095) {
    return result;
  }
  else
    return 0;
}
/********************************************************/  
          /* KOMPENZACE CHYB DAC PREVODNIKU */
/********************************************************/
/*  Vstupy  - PSU_channel = 1 nebo 2
            - variable = 'U' nebo 'I' 
            - data = hodnota odesilana do DAC 0-4095
    Vystup  - int hodnota kompenzovana kalibracimi konstantami
*/
int DA_kompenzace(uint8_t PSU_channel, char variable, int data) {
  //Serial.print("data: "); Serial.print(data); Serial.print("   CH:"); Serial.print(PSU_channel); Serial.print("   variable:"); Serial.println(variable);
  int result = 0;
    //Kompenzace napeti
  if(variable == 'U') {
    uint8_t index = data/100;
    if(data >= V_max/V_per_bit) {             //Vmax = 30V a V per bit = 0.01 => code 3000 je max, zastropovani
      result = data-DAC_u_calib_table[PSU_channel-1][(uint8_t)V_max];   //Index [30] pokud Vmax = 30 a na teto pozici je ulozena kompenzace pro kod 3000
    }
    else if(data <= DAC_code_low_limit) {     //Limitace min. nastavitelneho napeti operacnim zesilovacem
      result = data-DAC_u_calib_table[PSU_channel-1][0];
    }
    else {                                    //Linearni interpolace
      //  y = y1 + [(y2-y1)/(x2-x1)]*(x-x1)
      int interp_comp = DAC_u_calib_table[PSU_channel-1][index]+(((DAC_u_calib_table[PSU_channel-1][index+1]-DAC_u_calib_table[PSU_channel-1][index])/((index+1)*100-index*100))*(data-index*100));
      result = data-interp_comp;        //Napr. DAC=100, ADC=96, delta=-4, korigovane ADC=96-(-4)=100  
      //Serial.print("x:"); Serial.print(data); Serial.print("   x1:"); Serial.print(index*100); Serial.print("   x2:"); Serial.print((index+1)*100); Serial.print("   y1:"); Serial.print(DAC_u_calib_table[PSU_channel-1][index]); Serial.print("   y2:"); Serial.println(DAC_u_calib_table[PSU_channel-1][index+1]);
      //Serial.print("interp_comp:"); Serial.println(interp_comp);
    }
  }
    //Kompenzace proudu (nulovani)
  else if(variable == 'I') {
    return data;
  }
  //Serial.print("Result:"); Serial.println(result);
  //Serial.println("_____________________________");
  return result;
}
